/*  linux_ptx  - Linux specific library functions
 *  This file contains the common functions that are unique to
 *  Linux. The entire file is ignored in case of Windows
 *  
 *  Version 0.1
 *  
 *  Control PTZ-2N Camera
 *  
 *  Copyright 2005, Kenneth Lavrsen
 *  This program is published under the GNU General Public license
 */

#ifndef WIN32
#define DEBUG 0

#include <errno.h>
#include "linux_ptz.h"

/********************************************************************
 * open_device, Linux version
 *
 * Input:   serdevice name (/dev/tty0, /dev/tty1 etc)
 * 
 * Returns: Handle to the camera (type CAMERA)
 *
 ********************************************************************/
CAMERA open_device(char *device)
{
	CAMERA serdevice;
	struct termios adtio;

	//Setup serial port

	if ((serdevice = open(device, O_RDWR | O_NOCTTY)) < 0)
	{
		printf("\nUnable to open serial device %s\n", device);
		exit(0);
	}

	
	//We want full control of what is set and simply reset the entire adtio struct
	memset(&adtio, 0, sizeof(adtio));
		
	// Serial control options
	// First 5 flags: 8O1
	adtio.c_cflag |= PARENB;       // Parity
	adtio.c_cflag |= PARODD;       // And parity is odd	
	adtio.c_cflag &= ~CSTOPB;      // One stop bit
	adtio.c_cflag &= ~CSIZE;       // Character size mask
	adtio.c_cflag |= CS8;          // Character size 8 bits
	
	adtio.c_cflag |= CREAD;        // Enable Receiver ok
	adtio.c_cflag &= ~HUPCL;       // No "hangup"
	adtio.c_cflag &= ~CRTSCTS;     // No flowcontrol
	adtio.c_cflag |= CLOCAL;       // Local line - do not change "owner" of port ok

	// Baudrate, for newer systems
	cfsetispeed(&adtio, BAUDRATE);
	cfsetospeed(&adtio, BAUDRATE);	
	
	// Serial local options: adtio.c_lflag
	// Raw input = clear ICANON, ECHO, ECHOE, and ISIG
	// Disable misc other local features = clear FLUSHO, NOFLSH, TOSTOP, PENDIN, and IEXTEN
	// So we actually clear all flags in adtio.c_lflag
	adtio.c_lflag = 0;

	// Serial input options: adtio.c_iflag
	// Enable parity check = set INPCK and ISTRIP
	// Do not mark parity errors = clear PARMRK  
	// Disable software flow control = clear IXON, IXOFF, and IXANY
	// Disable any translation of CR and LF = clear INLCR, IGNCR, and ICRNL	
	// Ignore break condition on input = set IGNBRK
	// Ignore parity errors just in case = set IGNPAR;
	// So we can clear all flags except IGNBRK and IGNPAR
	adtio.c_iflag |= (INPCK | ISTRIP);
	
	// Serial output options
	// Raw output should disable all other output options
	adtio.c_oflag &= ~OPOST;

	adtio.c_cc[VTIME] = 10;		// timer 1s
	adtio.c_cc[VMIN] = 0;		// blocking read until 1 char
	
	if (tcsetattr(serdevice, TCSANOW, &adtio) < 0)
	{
		printf("Unable to initialize serial device");
		exit(0);
	}

	tcflush(serdevice, TCIOFLUSH);

	return serdevice;
}

/********************************************************************
 * close_device, Linux version
 *
 * Input: Handle to the weatherstation (type CAMERA)
 *
 * Returns nothing
 *
 ********************************************************************/
void close_device(CAMERA serdevice)
{
	close(serdevice);
	return;
}

/********************************************************************
 * init_camera PTZ2N (Linux version)
 * 
 * Input:   device number of the already open serial port
 *           
 * Returns: nothing, exits program if failing to reset
 *
 ********************************************************************/
void init_camera(CAMERA serdevice)
{
	char initstring1[] = "\x02"     "ckk"     "\x04";
	char initstring2[] = "\x02"    "cki0n"    "\x04";
	char initstring3[] = "\x02" "ckd0" "\xF2" "\x04";
	char initstring4[] = "\x02"    "ckz0m"    "\x04";
	//char readbuffer[20];
	//int readbytes, i;
	
	//i = strlen(initstring1);
	//printf("Stringlength = %d\n",i);
	
	write(serdevice, initstring1, strlen(initstring1));
	sleep(1);
	write(serdevice, initstring2, strlen(initstring2));
	sleep(1);
	//readbytes = read_device(serdevice, readbuffer, 5);
	//printf("Read %d bytes\n", readbytes);
	write(serdevice, initstring3, strlen(initstring3));
	sleep(1);
	write(serdevice, initstring4, strlen(initstring4));
	sleep(4);	
	
	return;
}

/********************************************************************
 * read_device in the Linux version is identical
 * to the standard Linux read() 
 *
 * Inputs:  serdevice - opened file handle
 *          buffer - pointer to the buffer to read into (unsigned char)
 *          size - number of bytes to read
 *
 * Output:  *buffer - modified on success (pointer to unsigned char)
 * 
 * Returns: number of bytes read
 *
 ********************************************************************/
int read_device(CAMERA serdevice, unsigned char *buffer, int size)
{
	int ret;

	for (;;) {
		ret = read(serdevice, buffer, size);
		if (ret == 0 && errno == EINTR)
			continue;
		return ret;
	}
}

/********************************************************************
 * write_device in the Linux version is identical
 * to the standard Linux write()
 *
 * Inputs:  serdevice - opened file handle
 *          buffer - pointer to the buffer to write from
 *          size - number of bytes to write
 *
 * Returns: number of bytes written
 *
 ********************************************************************/
int write_device(CAMERA serdevice, unsigned char *buffer, int size)
{
	int ret = write(serdevice, buffer, size);
	tcdrain(serdevice);	// wait for all output written
	return ret;
}

/********************************************************************
 * sleep_short - Linux version
 * 
 * Inputs: Time in milliseconds (integer)
 *
 * Returns: nothing
 *
 ********************************************************************/
void sleep_short(int milliseconds)
{
	usleep(milliseconds/1000);
}

/********************************************************************
 * sleep_long - Linux version
 * 
 * Inputs: Time in seconds (integer)
 *
 * Returns: nothing
 *
 ********************************************************************/
void sleep_long(int seconds)
{
	sleep(seconds);
}


/********************************************************************
 * http_request_url - Linux version
 * 
 * Inputs: urlline - URL to Motion control port with path and data
 *                   as a pointer to char array (string)
 *
 * Returns: 0 on success and -1 if fail.
 *
 * Action: Send a http request to Motion
 *
 ********************************************************************/
int http_request_url(char *urlline)
{
	int sockfd;
	struct hostent *hostinfo;
	struct sockaddr_in urladdress;
	char buffer[1024];
	int bytes_read;
	
	if ( (hostinfo = gethostbyname(MOTION_BASEURL)) == NULL )
	{
		perror("Host not known by DNS server or DNS server not working");
		return(-1);
	}
	
	if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0 )
	{
		perror("Cannot open socket");
		return(-1);
	}

	memset(&urladdress, 0, sizeof(urladdress));
	urladdress.sin_family = AF_INET;
	urladdress.sin_port = htons(MOTION_CONTROL_PORT);

	urladdress.sin_addr = *(struct in_addr *)*hostinfo->h_addr_list;

	if (connect(sockfd,(struct sockaddr*)&urladdress,sizeof(urladdress)) != 0)
	{
		perror("Cannot connect to host");
		return(-1);
	}
	sprintf(buffer, "GET %s\nHTTP/1.0\n\n", urlline);
	send(sockfd, buffer, strlen(buffer), 0);

	/* While there's data, read and print it */
	do
	{
		memset(buffer, 0, sizeof(buffer));
		bytes_read = recv(sockfd, buffer, sizeof(buffer), 0);
		if ( bytes_read > 0 )
			if (DEBUG) printf("%s", buffer);
	}
	while ( bytes_read > 0 );

	/* Close socket and clean up winsock */
	close(sockfd);
	
	return(0);
}

#endif
